import pytest
import tempfile
import os

from toolbox_iop.json_check_tools import count_json_list_length, find_duplicate_ids


def test_count_json_list_length_with_list():
    test_list = [{"id": 1}, {"id": 2}, {"id": 3}]
    result = count_json_list_length(test_list)
    assert result == 3


def test_count_json_list_length_with_empty_list():
    test_list = []
    result = count_json_list_length(test_list)
    assert result == 0


def test_count_json_list_length_with_dict():
    test_dict = {"id": 1, "name": "test"}
    result = count_json_list_length(test_dict)
    assert result is None


def test_count_json_list_length_with_string():
    test_string = "test string"
    result = count_json_list_length(test_string)
    assert result is None


def test_count_json_list_length_with_number():
    test_number = 42
    result = count_json_list_length(test_number)
    assert result is None


def test_count_json_list_length_with_nested_list():
    test_list = [[1, 2], [3, 4], [5, 6]]
    result = count_json_list_length(test_list)
    assert result == 3


def test_find_duplicate_ids_with_duplicates():
    test_data = [
        {"id": 1, "name": "item1"},
        {"id": 2, "name": "item2"},
        {"id": 1, "name": "item1_dup"},
        {"id": 3, "name": "item3"},
        {"id": 2, "name": "item2_dup"},
    ]
    
    with tempfile.NamedTemporaryFile(mode="w", suffix=".jsonl", delete=False) as f:
        for item in test_data:
            f.write(f"{item}\n")
        temp_file = f.name
    
    try:
        duplicates = find_duplicate_ids(temp_file, key="id", json_file=False)
        assert len(duplicates) == 2
        assert 1 in duplicates
        assert 2 in duplicates
        assert len(duplicates[1]) == 2
        assert len(duplicates[2]) == 2
    finally:
        os.unlink(temp_file)


def test_find_duplicate_ids_without_duplicates():
    test_data = [
        {"id": 1, "name": "item1"},
        {"id": 2, "name": "item2"},
        {"id": 3, "name": "item3"},
    ]
    
    with tempfile.NamedTemporaryFile(mode="w", suffix=".jsonl", delete=False) as f:
        for item in test_data:
            f.write(f"{item}\n")
        temp_file = f.name
    
    try:
        duplicates = find_duplicate_ids(temp_file, key="id", json_file=False)
        assert len(duplicates) == 0
    finally:
        os.unlink(temp_file)


def test_find_duplicate_ids_with_custom_key():
    test_data = [
        {"name": "item1"},
        {"name": "item2"},
        {"name": "item1"},
    ]
    
    with tempfile.NamedTemporaryFile(mode="w", suffix=".jsonl", delete=False) as f:
        for item in test_data:
            f.write(f"{item}\n")
        temp_file = f.name
    
    try:
        duplicates = find_duplicate_ids(temp_file, key="name", json_file=False)
        assert len(duplicates) == 1
        assert "item1" in duplicates
        assert len(duplicates["item1"]) == 2
    finally:
        os.unlink(temp_file)



from collections import defaultdict

from .json_tools import load_json, load_json_file


def find_duplicate_ids(jsonl_file, key="id", json_file=False):
    # 用于记录每个 id 出现的行号
    id_to_lines = defaultdict(list)
    if json_file:
        for x in load_json_file(jsonl_file):
            if key in x:
                id_to_lines[x[key]].append(x)

    else:
        load_json(jsonl_file, lambda x: id_to_lines[x[key]].append(x) if key in x else None)


    # 筛选出重复的 id
    duplicates = {
        id_val: lines for id_val, lines in id_to_lines.items() if len(lines) > 1
    }

    return duplicates


def count_json_list_length(json_obj):
    if isinstance(json_obj, list):
        return len(json_obj)
    return None




from toolbox_iop.json_check_tools import find_duplicate_ids


def parse_args():
    import argparse

    parser = argparse.ArgumentParser(description="检查JSONL文件中的重复ID")
    parser.add_argument("jsonl_file", help="要检查的JSONL文件路径")
    parser.add_argument("--key", "-k", help="要检查的JSONL的键名", default="id")
    parser.add_argument(
        "--json", "-j", help="解析json文件 【list(dict)格式】", action="store_true"
    )
    return parser.parse_args()


def main():
    args = parse_args()
    duplicates = find_duplicate_ids(args.jsonl_file, args.key, args.json)

    if duplicates:
        print("❌发现重复的 id：")
        for id_val, lines in duplicates.items():
            print(id_val)
            print(f"id '{id_val}' 在以下行重复出现: {lines}")
    else:
        print("✅未发现重复的 id。")


if __name__ == "__main__":
    main()



import json

try:
    from rich.console import Console

    ccprint = Console().print
except ImportError:
    ccprint = print


def dict_printer(process_count_dict: dict, total=None, print_method=print):
    if total is None:
        total = sum(process_count_dict.values())
    for key, value in process_count_dict.items():
        print()
        if isinstance(value, int):
            print_method(f"{key} {value} {value / total:.2%}")
        elif isinstance(value, dict):
            reject_total = sum(value.values())
            if reject_total == 0:
                reject_total = 1
            for k, v in value.items():
                # print(key, k, v, f"{v/reject_total:.2%}")

                print_method(f"{key} {k} {v} {v / reject_total:.2%}")
        else:
            raise NotImplementedError(f"value type {type(value)} not implemented")


def replace_escape_chars(string):
    return string.replace("\\n", "\n").replace("\\t", "\t").replace("\\r", "\r")


def parse_str(words, key="", print_switch=True, prefix="\n\n\n\n\n"):
    try:
        # 解析JSON字符串
        #
        data = json.loads(words)
        data = data[key] if key != "" else data
        # 在控制台输出解析后的结果
        # print(json.dumps(data, indent=4,ensure_ascii=False))
        formatted_json = json.dumps(data, indent=4, ensure_ascii=False, sort_keys=True)

        formatted_json = replace_escape_chars(formatted_json)
        if print_switch:
            print(f"{prefix}解析后的JSON对象:")
            ccprint(formatted_json)
        # print(formatted_json)
    except json.JSONDecodeError as e:
        import traceback

        print("\n输入的字符串不是有效的JSON格式:\n", traceback.format_exc())



import argparse
import random
import traceback

from tqdm import tqdm

from toolbox_iop.print_tools import parse_str

# def convert_str_to_number(s):
#     if "." in s:
#         return float(s)
#     else:
#         return int(s)


def read_file(
    file_path,
    random_order=False,
    key="",
    auto_scroll=False,
    print_switch=True,
    number="1.0",
):
    number = eval(number)
    # print(number)
    # print(type(number))
    try:
        with open(file_path, "r", encoding="utf-8") as file:
            lines = file.readlines()

            # 去除两端的空白符号（包括换行符）
            lines = [line for line in lines if line.strip()]

            # 如果指定了随机顺序，则打乱顺序
            if random_order:
                random.shuffle(lines)

            if number < 1:
                lines = lines[: int(len(lines) * number)]
            elif number == 1 and isinstance(number, float):
                lines = lines[:]
            else:
                lines = lines[:number]

            for line in tqdm(lines):
                parse_str(line, key, print_switch, "")
                if not auto_scroll:
                    user_input = input("按Enter后继续, Exit退出：\n").strip()
                    if user_input.lower() == "exit":
                        print("程序已退出。")
                        break
                if print_switch:
                    print("\n" * 5)
    except KeyboardInterrupt:
        print("程序已退出。")
    except FileNotFoundError:
        print(f"文件 {file_path} 未找到。")
    except Exception as e:
        print(f"读取文件时发生错误: {e}")
        traceback.print_exc()
    print("完成！")


def main():
    # 设置命令行参数解析
    parser = argparse.ArgumentParser(description="读取文件并逐行处理")
    parser.add_argument("file_path", type=str, help="文件路径")
    parser.add_argument(
        "-r", "--random", action="store_true", help="是否随机顺序读取文件内容"
    )
    parser.add_argument(
        "-k", "--key", type=str, help="读取关键词，默认显示全部", default=""
    )
    parser.add_argument("-a", "--all", action="store_true", help="循环显示全部")
    parser.add_argument(
        "-n", "--number", type=str, help="读取数量，默认读取全部", default="1.0"
    )
    parser.add_argument("--print_off", action="store_false", help="不显示输出")

    args = parser.parse_args()

    print(args)

    # 调用函数处理文件
    read_file(
        args.file_path, args.random, args.key, args.all, args.print_off, args.number
    )


if __name__ == "__main__":
    main()



import os
import tempfile
from pathlib import Path

import pytest
from unittest.mock import patch

from toolbox_iop.cli.count_cli import (
    detect_and_count_jsonl,
    count_jsonl_lines_fast,
    parse_args,
    main,
)


@pytest.fixture
def fixtures_path():
    return Path(__file__).parent / "fixtures"


def test_detect_and_count_jsonl_with_jsonl_file(fixtures_path):
    jsonl_file = fixtures_path / "test_jsonl_10lines.jsonl"
    is_jsonl, count = detect_and_count_jsonl(str(jsonl_file))
    assert is_jsonl is True
    assert count == 10


def test_detect_and_count_jsonl_with_large_jsonl_file(fixtures_path):
    jsonl_file = fixtures_path / "test_jsonl_100lines.jsonl"
    is_jsonl, count = detect_and_count_jsonl(str(jsonl_file))
    assert is_jsonl is True
    assert count == 100


def test_detect_and_count_jsonl_with_json_list_file(fixtures_path):
    json_file = fixtures_path / "test_json_list.json"
    is_jsonl, count = detect_and_count_jsonl(str(json_file))
    assert is_jsonl is False
    assert count == 1


def test_detect_and_count_jsonl_with_json_dict_file(fixtures_path):
    json_file = fixtures_path / "test_json_dict.json"
    is_jsonl, count = detect_and_count_jsonl(str(json_file))
    assert is_jsonl is False
    assert count == 1


def test_detect_and_count_jsonl_with_empty_file(fixtures_path):
    empty_file = fixtures_path / "test_empty.json"
    is_jsonl, count = detect_and_count_jsonl(str(empty_file))
    assert is_jsonl is False
    assert count == 0


def test_detect_and_count_jsonl_with_single_line(fixtures_path):
    single_file = fixtures_path / "test_single_line.json"
    is_jsonl, count = detect_and_count_jsonl(str(single_file))
    assert is_jsonl is False
    assert count == 1


def test_count_jsonl_lines_fast(fixtures_path):
    jsonl_file = fixtures_path / "test_jsonl_10lines.jsonl"
    count = count_jsonl_lines_fast(str(jsonl_file))
    assert count == 10


def test_count_jsonl_lines_fast_large_file(fixtures_path):
    jsonl_file = fixtures_path / "test_jsonl_100lines.jsonl"
    count = count_jsonl_lines_fast(str(jsonl_file))
    assert count == 100


def test_parse_args_with_file_path():
    with patch("sys.argv", ["toolbox-iop-count", "test.json"]):
        args = parse_args()
        assert args.file_path == "test.json"
        assert args.json is False
        assert args.jsonl is False


def test_parse_args_with_json_flag():
    with patch("sys.argv", ["toolbox-iop-count", "test.json", "--json"]):
        args = parse_args()
        assert args.file_path == "test.json"
        assert args.json is True
        assert args.jsonl is False


def test_parse_args_with_json_short_flag():
    with patch("sys.argv", ["toolbox-iop-count", "test.json", "-j"]):
        args = parse_args()
        assert args.file_path == "test.json"
        assert args.json is True
        assert args.jsonl is False


def test_parse_args_with_jsonl_flag():
    with patch("sys.argv", ["toolbox-iop-count", "test.jsonl", "--jsonl"]):
        args = parse_args()
        assert args.file_path == "test.jsonl"
        assert args.json is False
        assert args.jsonl is True


def test_parse_args_with_jsonl_short_flag():
    with patch("sys.argv", ["toolbox-iop-count", "test.jsonl", "-l"]):
        args = parse_args()
        assert args.file_path == "test.jsonl"
        assert args.json is False
        assert args.jsonl is True


@patch("sys.argv", ["toolbox-iop-count", "test.json", "--json"])
def test_main_with_json_flag_and_list_file(fixtures_path, capsys):
    json_file = fixtures_path / "test_json_list.json"
    
    with patch("toolbox_iop.cli.count_cli.parse_args") as mock_parse:
        mock_parse.return_value = type("Args", (), {
            "file_path": str(json_file),
            "json": True,
            "jsonl": False
        })()
        
        main()
        captured = capsys.readouterr()
        assert "✅JSON 是 list 类型，长度为: 5" in captured.out


@patch("sys.argv", ["toolbox-iop-count", "test.json", "--json"])
def test_main_with_json_flag_and_dict_file(fixtures_path, capsys):
    json_file = fixtures_path / "test_json_dict.json"
    
    with patch("toolbox_iop.cli.count_cli.parse_args") as mock_parse:
        mock_parse.return_value = type("Args", (), {
            "file_path": str(json_file),
            "json": True,
            "jsonl": False
        })()
        
        main()
        captured = capsys.readouterr()
        assert "❌JSON 不是 list 类型" in captured.out


@patch("sys.argv", ["toolbox-iop-count", "test.jsonl", "--jsonl"])
def test_main_with_jsonl_flag(fixtures_path, capsys):
    jsonl_file = fixtures_path / "test_jsonl_10lines.jsonl"
    
    with patch("toolbox_iop.cli.count_cli.parse_args") as mock_parse:
        mock_parse.return_value = type("Args", (), {
            "file_path": str(jsonl_file),
            "json": False,
            "jsonl": True
        })()
        
        main()
        captured = capsys.readouterr()
        assert "✅JSONL 文件，行数为: 10" in captured.out


@patch("sys.argv", ["toolbox-iop-count", "test.jsonl"])
def test_main_auto_detect_jsonl(fixtures_path, capsys):
    jsonl_file = fixtures_path / "test_jsonl_10lines.jsonl"
    
    with patch("toolbox_iop.cli.count_cli.parse_args") as mock_parse:
        mock_parse.return_value = type("Args", (), {
            "file_path": str(jsonl_file),
            "json": False,
            "jsonl": False
        })()
        
        main()
        captured = capsys.readouterr()
        assert "✅JSONL 文件，行数为: 10" in captured.out


@patch("sys.argv", ["toolbox-iop-count", "test.json"])
def test_main_auto_detect_json_list(fixtures_path, capsys):
    json_file = fixtures_path / "test_json_list.json"
    
    with patch("toolbox_iop.cli.count_cli.parse_args") as mock_parse:
        mock_parse.return_value = type("Args", (), {
            "file_path": str(json_file),
            "json": False,
            "jsonl": False
        })()
        
        main()
        captured = capsys.readouterr()
        assert "✅JSON 是 list 类型，长度为: 5" in captured.out


@patch("sys.argv", ["toolbox-iop-count", "test.json"])
def test_main_auto_detect_json_dict(fixtures_path, capsys):
    json_file = fixtures_path / "test_json_dict.json"
    
    with patch("toolbox_iop.cli.count_cli.parse_args") as mock_parse:
        mock_parse.return_value = type("Args", (), {
            "file_path": str(json_file),
            "json": False,
            "jsonl": False
        })()
        
        main()
        captured = capsys.readouterr()
        assert "❌JSON 不是 list 类型" in captured.out



from .json_check_tools import find_duplicate_ids
from .json_tools import load_json, save_normal_json, load_json_file, save_json_file
from .logger_tools import get_logger
from .print_tools import dict_printer
from .string_tools import normalize_newlines, remove_unwanted_tags


def main() -> None:
    print("Hello from toolbox-iop!")



import argparse

from toolbox_iop.json_check_tools import count_json_list_length
from toolbox_iop.json_tools import load_json, load_json_file, load_json_line


def detect_and_count_jsonl(file_path, sample_lines=10):
    with open(file_path, "r", encoding="utf-8") as f:
        lines = []
        line_count = 0
        total_lines_read = 0
        
        for line in f:
            total_lines_read += 1
            stripped = line.strip()
            if stripped:
                lines.append(stripped)
                line_count += 1
                if line_count >= sample_lines:
                    break
        
        if line_count == 0:
            return False, 0
        
        if line_count == 1:
            return False, 1
        
        try:
            load_json_line(lines[0])
            if line_count >= sample_lines:
                f.seek(0)
                total_lines = sum(1 for line in f if line.strip())
                return True, total_lines
            return True, line_count
        except:
            return False, line_count


def count_jsonl_lines_fast(file_path):
    with open(file_path, "r", encoding="utf-8") as f:
        return sum(1 for line in f if line.strip())


def parse_args():
    parser = argparse.ArgumentParser(description="统计JSON/JSONL文件中list的长度")
    parser.add_argument("file_path", help="要检查的JSON/JSONL文件路径")
    parser.add_argument(
        "--json", "-j", help="强制解析为json文件 【list(dict)格式】", action="store_true"
    )
    parser.add_argument(
        "--jsonl", "-l", help="强制解析为jsonl文件", action="store_true"
    )
    return parser.parse_args()


def main():
    args = parse_args()

    if args.json:
        json_obj = load_json_file(args.file_path)
        length = count_json_list_length(json_obj)

        if length is not None:
            print(f"✅JSON 是 list 类型，长度为: {length}")
        else:
            print("❌JSON 不是 list 类型")
    elif args.jsonl:
        count = count_jsonl_lines_fast(args.file_path)
        print(f"✅JSONL 文件，行数为: {count}")
    else:
        is_jsonl, count = detect_and_count_jsonl(args.file_path)
        
        if is_jsonl:
            print(f"✅JSONL 文件，行数为: {count}")
        else:
            if count == 1:
                json_obj = load_json_file(args.file_path)
                length = count_json_list_length(json_obj)

                if length is not None:
                    print(f"✅JSON 是 list 类型，长度为: {length}")
                else:
                    print("❌JSON 不是 list 类型")
            else:
                print(f"❌无法识别文件格式，非空行数: {count}")


if __name__ == "__main__":
    main()



import json
import os
from pathlib import Path


def generate_test_data():
    fixtures_dir = Path(__file__).parent / "fixtures"
    fixtures_dir.mkdir(exist_ok=True)
    
    test_json_list = [
        {"id": 1, "name": "item1"},
        {"id": 2, "name": "item2"},
        {"id": 3, "name": "item3"},
        {"id": 4, "name": "item4"},
        {"id": 5, "name": "item5"}
    ]
    
    test_json_dict = {"id": 1, "name": "item1", "description": "This is a test item"}
    
    test_single_line = [{"id": 1, "name": "item1"}]
    
    test_jsonl_10lines = [{"id": i + 1, "name": f"item{i + 1}"} for i in range(10)]
    
    test_jsonl_100lines = [{"id": i + 1, "name": f"item{i + 1}"} for i in range(100)]
    
    with open(fixtures_dir / "test_json_list.json", "w", encoding="utf-8") as f:
        json.dump(test_json_list, f, ensure_ascii=False)
    
    with open(fixtures_dir / "test_json_dict.json", "w", encoding="utf-8") as f:
        json.dump(test_json_dict, f, ensure_ascii=False)
    
    with open(fixtures_dir / "test_single_line.json", "w", encoding="utf-8") as f:
        json.dump(test_single_line, f, ensure_ascii=False)
    
    with open(fixtures_dir / "test_empty.json", "w", encoding="utf-8") as f:
        f.write("")
    
    with open(fixtures_dir / "test_jsonl_10lines.jsonl", "w", encoding="utf-8") as f:
        for item in test_jsonl_10lines:
            f.write(json.dumps(item, ensure_ascii=False) + "\n")
    
    with open(fixtures_dir / "test_jsonl_100lines.jsonl", "w", encoding="utf-8") as f:
        for item in test_jsonl_100lines:
            f.write(json.dumps(item, ensure_ascii=False) + "\n")
    
    print(f"测试数据文件已生成到: {fixtures_dir}")
    print("生成的文件:")
    for file in fixtures_dir.glob("test_*"):
        print(f"  - {file.name}")


if __name__ == "__main__":
    generate_test_data()
